#include <Carbon/Carbon.h>
#include <ApplicationServices/ApplicationServices.h>
#include <CoreFoundation/CoreFoundation.h>

#include <vector>
#include <string>

const int MOVE_REL_DISTANCE = 16;
const int MENUBAR_HEIGHT = 22;

using namespace std;

enum SkipStrategy {
    SKIP_APPLE_EVENT, SKIP_NON_VISIBLE, ALL
};

struct Win {
    AXUIElementRef win;
    ProcessSerialNumber psn;
    pid_t pid;
    string title;
    bool focused, visible, front;
    int x, y, w, h;
    int cx, cy;

    Win(ProcessSerialNumber psn0, pid_t pid0, string title0, bool focused0,
        int x0, int y0, int w0, int h0);
    explicit Win(AXUIElementRef winRef);

    bool hasSameGeom(const Win& r);
    void dump() const;
};

class Mgr {
public:
    Mgr(int argc, char* argv[]);

    void dump();

    void switch_u();
    void switch_r();
    void switch_d();
    void switch_l();
    void switch_ur();
    void switch_dr();
    void switch_dl();
    void switch_ul();

    void switch_self();

    void move_ur();
    void move_dr();
    void move_dl();
    void move_ul();

    void move_u();
    void move_r();
    void move_d();
    void move_l();

    void maximize();
    void close();
    void iconify();

    void deiconify();

    vector<Win> windows();
    vector<Win> visibleWindows();

    int focusedWindow();

    void switch_dir(int x, int y);
    void switch_to(int index);

    void move_to(int wid, int x, int y);
    void move_rel(int x, int y);
    void move_side(int x, int y);

    void initWindows(SkipStrategy skipStrategy);

    void push_button(CFStringRef s);

private:
    void searchWindow(Win& w);
};
